<?php
namespace Ak\Fs;

use League\Flysystem\FilesystemInterface;

/**
 * Upravitelj datotekama.
 *
 * @package Ak\Fs\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/fs
 */
final class FileManager implements FileManagerInterface
{
    /** @var FilesystemInterface */
    private $fs;
    /** @var FileLoaderInterface[] */
    private $loaders;

    /**
     * Konstruktor.
     *
     * @param FilesystemInterface $fs
     * @param array $loaders
     */
    public function __construct(FilesystemInterface $fs, array $loaders)
    {
        $this->fs = $fs;
        $this->loaders = $loaders;
    }

    /**
     * Briše datoteku s datom putanjom.
     *
     * @param string $path
     * @throws \League\Flysystem\FileNotFoundException
     */
    public function delete(string $path): void
    {
        if ($this->exists($path)) {
            $this->fs->delete($path);
        }
    }

    /**
     * Vraća datoteku s datom putanjom.
     *
     * @param string $path
     * @return FileInterface
     */
    public function get(string $path): FileInterface
    {
        foreach ($this->loaders as $loader) {
            if ($file = $loader->load($path)) {
                return $file;
            }
        }

        return new File($path);
    }

    /**
     * Da li datoteka s datom putanjom postoji?
     *
     * @param string $path
     *
     * @return bool
     */
    public function exists(string $path): bool
    {
        return $this->fs->has($path);
    }

    /**
     * Zapisuje datu datoteku.
     *
     * @param FileInterface $file
     * @throws \League\Flysystem\FileExistsException
     * @throws \League\Flysystem\FileNotFoundException
     */
    public function write(FileInterface $file): void
    {
        if ($this->exists($file->getPath())) {
            $this->fs->update($file->getPath(), $file->getContents());
        } else {
            $this->fs->write($file->getPath(), $file->getContents());
        }
    }
}