<?php
namespace Ak\Fs\Service;

use Ak\Fs\Adapter\AdapterPluginManager;
use Interop\Container\ContainerInterface;
use League\Flysystem\Config;
use League\Flysystem\Filesystem;
use Laminas\ServiceManager\Exception;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Stvara novu instancu Filesystema.
 *
 * @package Ak\Fs
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/fs
 */
final class FilesystemAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Da li možemo stvoriti instancu traženog servisa?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        $config = $container->get('config')['fs'] ?? [];
        return array_key_exists($requestedName, $config);
    }

    /**
     * Stvara instancu.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return Filesystem
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $options ?: ($container->get('config')['fs'][$requestedName] ?? null);

        if (null === $config) {
            throw new Exception\ServiceNotFoundException(sprintf(
                "Filesystem s imenom '%s' ne postoji.", $requestedName
            ));
        }

        if (!($config['adapter'] ?? false)) {
            throw new Exception\ServiceNotCreatedException(sprintf(
                "Filesystem s imenom '%s' nema definiran adapter.", $requestedName
            ));
        }

        /** @var AdapterPluginManager $adapters */
        $adapters = $container->get(AdapterPluginManager::class);
        $adapter = $adapters->get($config['adapter'], $config['options'] ?? null);

        $filesystemClass = $config['filesystem'] ?? Filesystem::class;

        return new $filesystemClass($adapter, new Config([
            'disable_asserts' => true,
        ]));
    }
}
