<?php
namespace AkTest\Fs\Service;

use Ak\Fs\Adapter\AdapterPluginManager;
use Ak\Fs\Service\FilesystemAbstractFactory;
use AkTest\Fs\AbstractTestCase;
use AkTest\Fs\Service\Mock\MockFilesystem;
use League\Flysystem\AdapterInterface;
use League\Flysystem\Filesystem;
use PHPUnit\Framework\MockObject\MockObject;
use Zend\ServiceManager\PluginManagerInterface;

/**
 * Class FilesystemAbstractFactoryTest
 *
 * @package AkTest\Fs\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/fs
 */
final class FilesystemAbstractFactoryTest extends AbstractTestCase
{
    public function testCanCreateFsIfDefinedInConfig()
    {
        $container = $this->getContainer([
            'fs' => [
                'test' => [
                    'adapter' => 'null',
                ],
            ],
        ]);

        $factory = new FilesystemAbstractFactory();
        $this->assertTrue($factory->canCreate($container, 'test'));
    }

    public function testCannotCreateFsIfNotDefinedInConfig()
    {
        $container = $this->getContainer();

        $factory = new FilesystemAbstractFactory();
        $this->assertFalse($factory->canCreate($container, 'invalid'));
    }

    /**
     * @expectedException \Zend\ServiceManager\Exception\ServiceNotFoundException
     */
    public function testWillThrowExceptionIfServiceDoesntExist()
    {
        $container = $this->getContainer();

        $factory = new FilesystemAbstractFactory();
        $factory($container, 'invalid');
    }

    /**
     * @expectedException \Zend\ServiceManager\Exception\ServiceNotCreatedException
     */
    public function testWillThrowExceptionIfAdapterNotDefined()
    {
        $container = $this->getContainer([
            'fs' => [
                'no-adapter' => [],
            ],
        ]);

        $factory = new FilesystemAbstractFactory();
        $factory($container, 'no-adapter');
    }

    public function testWillUseTheCorrectAdapter()
    {
        $adapterOptions = ['root' => __DIR__];
        /** @var AdapterInterface|MockObject $adapter */
        $adapter = $this->createMock(AdapterInterface::class);
        /** @var PluginManagerInterface|MockObject $adapterManager */
        $adapterManager = $this->createMock(PluginManagerInterface::class);
        $adapterManager->expects($this->once())
            ->method('get')
            ->with('local', $adapterOptions)
            ->willReturn($adapter);

        $container = $this->getContainer([
            'dependencies' => [
                'services' => [
                    AdapterPluginManager::class => $adapterManager,
                ],
            ],

            'fs' => [
                'test' => [
                    'adapter' => 'local',
                    'options' => $adapterOptions,
                ],
            ],
        ]);

        $factory = new FilesystemAbstractFactory();
        /** @var Filesystem $fs */
        $fs = $factory($container, 'test');

        $this->assertSame($adapter, $fs->getAdapter());
    }

    public function testCanPassOptionsDirecty()
    {
        $adapterOptions = ['root' => __DIR__];
        /** @var AdapterInterface|MockObject $adapter */
        $adapter = $this->createMock(AdapterInterface::class);
        /** @var PluginManagerInterface|MockObject $adapterManager */
        $adapterManager = $this->createMock(PluginManagerInterface::class);
        $adapterManager->expects($this->once())
            ->method('get')
            ->with('local', $adapterOptions)
            ->willReturn($adapter);

        $container = $this->getContainer([
            'dependencies' => [
                'services' => [
                    AdapterPluginManager::class => $adapterManager,
                ],
            ],
        ]);

        $factory = new FilesystemAbstractFactory();
        /** @var Filesystem $fs */
        $fs = $factory($container, '', [
            'adapter' => 'local',
            'options' => $adapterOptions,
        ]);

        $this->assertSame($adapter, $fs->getAdapter());
    }

    public function testCanCreateCustomFilesystem()
    {
        $container = $this->getContainer();

        $factory = new FilesystemAbstractFactory();
        $fs = $factory($container, '', [
            'adapter' => 'local',
            'filesystem' => MockFilesystem::class,
            'options' => [
                'root' => dirname(__DIR__) . '/data',
            ],
        ]);

        $this->assertInstanceOf(MockFilesystem::class, $fs);
    }
}